#ifndef _Command_CPP
#define _Command_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <WindowsX.H>
#include <ShellAPI.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <SQL.H>
#include <SqlExt.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../Resources/Resource.H"

#include "../../SharedSource/LZARICode.H"
#include "../../SharedSource/RLECompression.H"

#include "../../SharedClasses/SQLClass/cSQL.H"
#include "../../SharedClasses/SQLClass/cRecordSet.H"
#include "../../SharedClasses/CMemPool/CMemPool.H"
#include "../../SharedClasses/LZSS/LZSS_Compress.H"
#include "../../SharedClasses/LZSS/LZSS_Uncompress.H"

#include "../CSockSrvr/CSockSrvr.H"

#include "../../SharedSource/Debug.H"
#include "../../SharedSource/NSWFL.H"
#include "../../SharedSource/Common.H"
#include "../../SharedSource/CRC32.H"

#include "Init.H"
#include "Routines.H"
#include "Command.H"
#include "SQLImport.H"
#include "Console.H"
#include "NewDB.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ReceiveFileData(CSockSrvr *pSockSrvr, int iClient, char *sFileName);

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	int ProcessCommand(CSockSrvr *pSockSrvr, int iClient, char *sRecvBuf, int iRecvBufSz);

	Processes a command from the peer.

	Possible return Values:
		CMD_OK    The command was a success, all is well.
		CMD_DONE  The command was a success, close the connection.
		CMD_ERROR The command failed due to an error.
*/
int ProcessCommand(CSockSrvr *pSockSrvr, int iClient, char *sCmdBuf, int iCmdBufSz)
{
    char sCmdData[SENDBUFSZ + RECVBUFSZ];

	char *sTempPtr = NULL;

	int iSendBufSz = 0;
	int iCmdDataSz = 0;
	int iCmdFlagLength = 0;

	//After the transfer of the database creation scripts (to create the database
	//	structure skeleton), the client will send the script CRC.
	if((iCmdFlagLength = CmdCmp(sCmdBuf, "::DBCreationCRC->")))
    {
		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);

		//Check the CRC.
		if(CCI[iClient].dwFileCRC != atol(sCmdData))
		{
			sTempPtr = "Received a corrupt DB creation script.";

			sprintf(sCmdData, "%s \"%s\". [%X-%X]", sTempPtr,
				CCI[iClient].sFileName, CCI[iClient].dwFileCRC, atol(sCmdData));
			WriteLogEx(pSockSrvr->icClientID[iClient], sCmdData, EVENT_ERROR);

			sprintf(sCmdData, "::Msg->%s", sTempPtr);
			pSockSrvr->SetNextSendData(iClient, sCmdData);

			return CMD_ERROR;
		}

		//Create the client database structure skeleton.
		if(CreateNewDBFromDBCreationScript(&CCI[iClient].cCustSQL, pSockSrvr, iClient, CCI[iClient].sFileName))
		{
			pSockSrvr->SetNextSendData(iClient, "::BeginReplication->All");
			return CMD_OK;
		}
        else{
			sTempPtr = "Failed to create the replication target database.";
			WriteLogEx(pSockSrvr->icClientID[iClient], sTempPtr, EVENT_ERROR);

			sprintf(sCmdData, "::Msg->%s", sTempPtr);
			pSockSrvr->SetNextSendData(iClient, sCmdData);
			return CMD_ERROR;
        }

		return CMD_OK;
	}
	//The client is ready to send the client database creation scripts.
	//	This is in response to a "::BeginInit" command.
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::SendingDBCreationSQL")))
    {
        sprintf(CCI[iClient].sFileName, "%s\\%s\\DBCreation.wpd", gsImportTempDir, CCI[iClient].sCompanyName);
        CreateFolderStructure(CCI[iClient].sFileName);

		if(ReceiveFileData(pSockSrvr, iClient, CCI[iClient].sFileName))
		{
			if(gbDebugMode)
			{
				sprintf(sCmdData, "Waiting on DB Creation SQL file CRC \"%s\"", CCI[iClient].sFileName);
				WriteLogEx(pSockSrvr->icClientID[iClient], sCmdData, EVENT_NONE);
			}

			return CMD_OK;
		}

		return CMD_ERROR;
	}
	//The client is reporting that all processing is complete and successful.
	//	This command is sent by the client at the end of a synchronization
	//	(initialized by server commands: "::BeginInit", "::BeginReplication->Transactions"
	//	if the synchronization was fully successful.
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::Complete")))
    {
		WriteLogEx(pSockSrvr->icClientID[iClient], "The process has been completed successfully.", EVENT_NONE);

		CCI[iClient].cIndexSQL.bThrowErrors = true;

		if(CCI[iClient].bRequestInit)
		{
			sprintf(sCmdData, "UPDATE [%s].[%s].[Companys]"
				" SET Init = 0"
				" WHERE CompName = '%s'",
				gsSQLIndexDatabase, gsDefaultDBO, CCI[iClient].sCompanyName);

			CCI[iClient].cIndexSQL.ExecuteNonQuery(sCmdData);
		}

		UpdateClientStats(pSockSrvr, iClient, &CCI[iClient].cIndexSQL);

		return CMD_DONE;
	}
	//This is a general processing message from the client.
	//	All messages are file logged and written to the event log.
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::Msg->")))
    {
		if(gbDebugMode)
		{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Received a message from the server.", EVENT_NONE);
		}

		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);
		WriteLogEx(pSockSrvr->icClientID[iClient], sCmdData, EVENT_INFO);
		return CMD_OK;
	}
	//After the transfer of the transactional data, the client will send the data file CRC.
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::TransDataCRC->")) ||
		(iCmdFlagLength = CmdCmp(sCmdBuf, "::TransDumpCRC->")))
    {
		/*
		bool bDumpData = false;

		if(strnicmp(sCmdBuf, "::TransDumpCRC->", iCmdFlagLength) == 0)
		{
			bDumpData = true;
		}
		else bDumpData = false;
		*/

		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);

		if(CCI[iClient].dwFileCRC != atol(sCmdData))
		{
			sprintf(sCmdData, "Received a corrupt transaction file \"%s\". [%X-%X]",
				CCI[iClient].sFileName, CCI[iClient].dwFileCRC, atol(sCmdData));
			WriteLogEx(pSockSrvr->icClientID[iClient], sCmdData, EVENT_ERROR);

			if(CCI[iClient].bRequestInit)
			{
				if(gbRTOFDWI)
				{
					if(CCI[iClient].dwCorruptInitRetry >= 5)
					{
						sprintf(sCmdData, "Failed to receive file after %d Unsuccessful retries. Failing!",
							CCI[iClient].dwCorruptInitRetry);
						WriteLogEx(pSockSrvr->icClientID[iClient], sCmdData, EVENT_ERROR);
						return CMD_ERROR;					
					}

					CCI[iClient].dwCorruptInitRetry++;

					sprintf(sCmdData, "Making request %d for corrupt data file.",
						CCI[iClient].dwCorruptInitRetry);
					WriteLogEx(pSockSrvr->icClientID[iClient], sCmdData, EVENT_WARN);
									
					sprintf(sCmdData, "::TransDataDatabase->%s", CCI[iClient].sTransDatabase);
					pSockSrvr->SetNextSendData(iClient, sCmdData);

					sprintf(sCmdData, "::TransDataTable->%s", CCI[iClient].sTransTable);
					pSockSrvr->SetNextSendData(iClient, sCmdData);

					sprintf(sCmdData, "::TransDataDBO->%s", CCI[iClient].sTransDBO);
					pSockSrvr->SetNextSendData(iClient, sCmdData);

					pSockSrvr->SetNextSendData(iClient, "::DumpTable");
					return CMD_OK;
				}
				else{
					pSockSrvr->SetNextSendData(iClient, "::LastImportFailed");
					// FIXME: Should we return CMD_OK or CMD_ERROR?
					return CMD_ERROR;
				}
			}
			else{
				pSockSrvr->SetNextSendData(iClient, "::LastImportFailed");
				// FIXME: Should we return CMD_OK or CMD_ERROR?
				return CMD_OK;
			}
		}

		bool bCheckExistance = true;

		if(CCI[iClient].bRequestInit)
		{
			bCheckExistance = false;
		}

		CCI[iClient].dwCorruptInitRetry = 0;

		if(ImportSQLResults(pSockSrvr, iClient, bCheckExistance, CCI[iClient].sFileName))
        {
			pSockSrvr->SetNextSendData(iClient, "::LastImportSuccess");
			return CMD_OK;
        }
        else{
            // FIXME: Should we return CMD_OK or CMD_ERROR?
			pSockSrvr->SetNextSendData(iClient, "::LastImportFailed");
			return CMD_ERROR;
        }

		return CMD_OK;
	}
	//Receive the Transaction Data File size.
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::TransDataFileSize->")))
    {
		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);
		CCI[iClient].dwTransFileSize = atol(sCmdData);
		return CMD_OK;
	}
	//Receive the Transaction Data File database.
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::TransDataDatabase->")))
    {
		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);
		strcpy(CCI[iClient].sTransDatabase, sCmdData);
		return CMD_OK;
	}
	//Receive the Transaction Data File table.
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::TransDataTable->")))
    {
		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);
		strcpy(CCI[iClient].sTransTable, sCmdData);
		return CMD_OK;
	}
	//Receive the Transaction Data File default DBO.
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::TransDataDBO->")))
    {
		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);
		strcpy(CCI[iClient].sTransDBO, sCmdData);
		return CMD_OK;
	}
	//The "::SendingTransData" command is send when the client ready
	//	to send the transactional data file. This is either in response
	//	to a "::DumpTable" command or part of the transactional or
	//	full synchronization commands.
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::SendingTransData")))
    {
		float fSize = (float) CCI[iClient].dwTransFileSize;
		
		sprintf(sCmdData, "Receiving transaction data for [%s].[%s].[%s] (%.3f KB)",
			CCI[iClient].sTransDatabase,
			CCI[iClient].sTransDBO,
			CCI[iClient].sTransTable,
			fSize / 1024);
		WriteLogEx(pSockSrvr->icClientID[iClient], sCmdData, EVENT_NONE);

		sprintf(CCI[iClient].sFileName, "%s\\%s\\%s\\%s.wpd",
			gsImportTempDir,
			CCI[iClient].sCompanyName,
			CCI[iClient].sTransDatabase, 
			CCI[iClient].sTransTable);

        CreateFolderStructure(CCI[iClient].sFileName);

		if(ReceiveFileData(pSockSrvr, iClient, CCI[iClient].sFileName))
		{
			return CMD_OK;
		}

		return CMD_ERROR;
	}
/*
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::SendingFileData->")))
    {
		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);

        sprintf(CCI[iClient].sFileName, "%s\\%s\\%s.wpd", gsImportTempDir, CCI[iClient].sCompanyName, sCmdData);
        CreateFolderStructure(CCI[iClient].sFileName);

		if(ReceiveFileData(pSockSrvr, iClient, CCI[iClient].sFileName))
		{
			return CMD_OK;
		}

		return CMD_ERROR;
	}
*/
	WriteLogEx(pSockSrvr->icClientID[iClient], "Received unknown command.", EVENT_ERROR);

	sCmdBuf[1024] = '\0';
	WriteLogEx(pSockSrvr->icClientID[iClient], sCmdBuf, EVENT_ERROR);
	return CMD_ERROR;	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool UpdateClientStats(CSockSrvr *pSockSrvr, int iClient, CSQL *lpSQL)
{
	SYSTEMTIME ST;

	char sSQL[2048];
	char lsDate[64];
	char lsTime[64];

	char sConnectTime[128];
	char sDisconnectTime[128];

	GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &CCI[iClient].stConnectTime, NULL, lsDate, sizeof(lsDate));
	GetTimeFormat(LOCALE_USER_DEFAULT, TIME_FORCE24HOURFORMAT, &CCI[iClient].stConnectTime, NULL, lsTime, sizeof(lsTime));
	sprintf(sConnectTime, "%s %s", lsDate, lsTime);

	GetSystemTime(&ST);
	GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &ST, NULL, lsDate, sizeof(lsDate));
	GetTimeFormat(LOCALE_USER_DEFAULT, TIME_FORCE24HOURFORMAT, &ST, NULL, lsTime, sizeof(lsTime));
	sprintf(sDisconnectTime, "%s %s", lsDate, lsTime);


	sprintf(sSQL, "INSERT INTO [%s].[%s].[CompanyStats]"
		" (Company_Key, ConnectTime, DisconnectTime, BytesSent, BytesReceived)"
		" select IsNull((select TOP 1 ID from Companys where CompName = '%s'), -1),"
		" '%s', '%s', %d, %d",
		gsSQLIndexDatabase, gsDefaultDBO, CCI[iClient].sCompanyName,
		sConnectTime, sDisconnectTime, CCI[iClient].dwBytesSent, CCI[iClient].dwBytesRecv);

	lpSQL->ExecuteNonQuery(sSQL);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ReceiveFileData(CSockSrvr *pSockSrvr, int iClient, char *sFileName)
{
    FILE *hTargetHandle = NULL;

    char sText[1024];
	char sFullFile[MAX_PATH];
	char sRecvBuf[RECVBUFSZ + 1];
    int iRecvBufSz = 0;

	DWORD dwDataCRC = 0xffffffff;

	WriteLogEx(pSockSrvr->icClientID[iClient], "Begining file transfer.", EVENT_NONE);

	if(strcmpi(gsCompressionMethod, "None") != 0)
	{
		sprintf(sFullFile, "%s.sec", sFileName);
	}
	else strcpy(sFullFile, sFileName);

	if( (hTargetHandle = fopen(sFullFile, "wb")) == NULL)
    {
		WriteLogEx(pSockSrvr->icClientID[iClient], "Failed to open export file for binary write.", EVENT_ERROR);
        return false;
    }

	if(gbDebugMode)
	{
		sprintf(sText, "File opened successfully. [%s]", sFullFile);
		WriteLogEx(pSockSrvr->icClientID[iClient], sText, EVENT_NONE);
	}

	while(pSockSrvr->bcConnected[iClient] && !pSockSrvr->bcDisconnect[iClient])
    {
		if(pSockSrvr->GetClientData(iClient, sRecvBuf, &iRecvBufSz))
        {
			if(iRecvBufSz == 5)
			{
				sRecvBuf[iRecvBufSz] = '\0';
				if(strcmp(sRecvBuf, "::EOF") == 0)
				{
					break; // All is well.
				}
			}

			Decode(&CCI[iClient].SC, sRecvBuf, sRecvBuf, iRecvBufSz);
            dwDataCRC = PartialCRC(dwDataCRC, sRecvBuf, iRecvBufSz);
			fwrite(sRecvBuf, sizeof(char), iRecvBufSz, hTargetHandle);
		}
		else Sleep(1);
	}

	dwDataCRC = (dwDataCRC ^ 0xffffffff);
	CCI[iClient].dwFileCRC = dwDataCRC;

	sprintf(sRecvBuf, "Received data file. CRC: %X", dwDataCRC);
	WriteLogEx(pSockSrvr->icClientID[iClient], sRecvBuf, EVENT_NONE);

	fclose(hTargetHandle);

	if(strcmpi(gsCompressionMethod, "None") != 0)
	{
		float dwBeforeSz = 0;
		float dwAfterSz = 0;

		sprintf(sFullFile, "%s.sec", sFileName);

		dwBeforeSz = (float) Get_FileSize(sFullFile);

		sprintf(sRecvBuf, "Decompressing file. (%.2f KB)", dwBeforeSz / 1024);
		WriteLogEx(pSockSrvr->icClientID[iClient], sRecvBuf, EVENT_NONE);

		if(strcmpi(gsCompressionMethod, "LZSS") == 0){
			LZSSDeCompressFile(sFullFile, sFileName);
		}
		else if(strcmpi(gsCompressionMethod, "LZARI") == 0){
			DecompressFile(sFullFile, sFileName);
		}
		else if(strcmpi(gsCompressionMethod, "RLE") == 0){
			RLEDecodeFile(sFullFile, sFileName);			
		}

		dwAfterSz = (float) Get_FileSize(sFileName);

		sprintf(sRecvBuf, "File size increased from %.2f KB to %.2f KB. (%.2f %% Decompression).",
			dwBeforeSz / 1024, dwAfterSz / 1024, (100 - ((dwBeforeSz / dwAfterSz) * 100)));
		WriteLogEx(pSockSrvr->icClientID[iClient], sRecvBuf, EVENT_NONE);

		if(gbDeleteCompressedFiles)
		{
			DeleteFile(sFullFile);
		}
	}

	if(pSockSrvr->bcConnected[iClient] && !pSockSrvr->bcDisconnect[iClient])
	{
		return true;
	}

	WriteLogEx(pSockSrvr->icClientID[iClient], "Failed to receive client data. Client disconncted.", EVENT_ERROR);

	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
